@@ -14,11 +14,7 @@
 ** testing of the SQLite library.
 */
 #include "sqliteInt.h"
-#if defined(INCLUDE_SQLITE_TCL_H)
-#  include "sqlite_tcl.h"
-#else
-#  include "tcl.h"
-#endif
+#include "tcl.h"
 #include <stdlib.h>
 #include <string.h>
 
@@ -29,7 +25,7 @@ typedef struct echo_cursor echo_cursor;
 
 /*
 ** The test module defined in this file uses four global Tcl variables to
-** communicate with test-scripts:
+** commicate with test-scripts:
 **
 **     $::echo_module
 **     $::echo_module_sync_fail
@@ -196,7 +192,7 @@ static int getColumnNames(
         rc = SQLITE_NOMEM;
         goto out;
       }
-      nBytes += (int)strlen(zName)+1;
+      nBytes += strlen(zName)+1;
     }
     aCol = (char **)sqlite3MallocZero(nBytes);
     if( !aCol ){
@@ -210,8 +206,8 @@ static int getColumnNames(
     zSpace = (char *)(&aCol[nCol]);
     for(ii=0; ii<nCol; ii++){
       aCol[ii] = zSpace;
-      sqlite3_snprintf(nBytes, zSpace, "%s", sqlite3_column_name(pStmt,ii));
-      zSpace += (int)strlen(zSpace) + 1;
+      zSpace += sprintf(zSpace, "%s", sqlite3_column_name(pStmt, ii));
+      zSpace++;
     }
     assert( (zSpace-nBytes)==(char *)aCol );
   }
@@ -269,7 +265,6 @@ static int getIndexArray(
   while( pStmt && sqlite3_step(pStmt)==SQLITE_ROW ){
     const char *zIdx = (const char *)sqlite3_column_text(pStmt, 1);
     sqlite3_stmt *pStmt2 = 0;
-    if( zIdx==0 ) continue;
     zSql = sqlite3_mprintf("PRAGMA index_info(%s)", zIdx);
     if( !zSql ){
       rc = SQLITE_NOMEM;
@@ -341,7 +336,7 @@ static int echoDeclareVtab(
   if( pVtab->zTableName ){
     sqlite3_stmt *pStmt = 0;
     rc = sqlite3_prepare(db, 
-        "SELECT sql FROM sqlite_schema WHERE type = 'table' AND name = ?",
+        "SELECT sql FROM sqlite_master WHERE type = 'table' AND name = ?",
         -1, &pStmt, 0);
     if( rc==SQLITE_OK ){
       sqlite3_bind_text(pStmt, 1, pVtab->zTableName, -1, 0);
@@ -389,7 +384,6 @@ static int echoDestructor(sqlite3_vtab *pVtab){
 typedef struct EchoModule EchoModule;
 struct EchoModule {
   Tcl_Interp *interp;
-  sqlite3 *db;
 };
 
 /*
@@ -491,7 +485,7 @@ static int echoCreate(
     rc = sqlite3_exec(db, zSql, 0, 0, 0);
     sqlite3_free(zSql);
     if( rc!=SQLITE_OK ){
-      *pzErr = sqlite3_mprintf("%s", sqlite3_errmsg(db));
+      *pzErr = sqlite3DbStrDup(0, sqlite3_errmsg(db));
     }
   }
 
@@ -653,12 +647,12 @@ static int echoRowid(sqlite3_vtab_cursor *cur, sqlite_int64 *pRowid){
 ** indeed the hash of the supplied idxStr.
 */
 static int hashString(const char *zString){
-  u32 val = 0;
+  int val = 0;
   int ii;
   for(ii=0; zString[ii]; ii++){
     val = (val << 3) + (int)zString[ii];
   }
-  return (int)(val&0x7fffffff);
+  return val;
 }
 
 /* 
@@ -750,34 +744,6 @@ static void string_concat(char **pzStr, char *zAppend, int doFree, int *pRc){
   }
 }
 
-/*
-** This function returns a pointer to an sqlite3_malloc()ed buffer 
-** containing the select-list (the thing between keywords SELECT and FROM)
-** to query the underlying real table with for the scan described by
-** argument pIdxInfo.
-**
-** If the current SQLite version is earlier than 3.10.0, this is just "*"
-** (select all columns). Or, for version 3.10.0 and greater, the list of
-** columns identified by the pIdxInfo->colUsed mask.
-*/
-static char *echoSelectList(echo_vtab *pTab, sqlite3_index_info *pIdxInfo){
-  char *zRet = 0;
-  if( sqlite3_libversion_number()<3010000 ){
-    zRet = sqlite3_mprintf(", *");
-  }else{
-    int i;
-    for(i=0; i<pTab->nCol; i++){
-      if( pIdxInfo->colUsed & ((sqlite3_uint64)1 << (i>=63 ? 63 : i)) ){
-        zRet = sqlite3_mprintf("%z, %s", zRet, pTab->aCol[i]);
-      }else{
-        zRet = sqlite3_mprintf("%z, NULL", zRet);
-      }
-      if( !zRet ) break;
-    }
-  }
-  return zRet;
-}
-
 /*
 ** The echo module implements the subset of query constraints and sort
 ** orders that may take advantage of SQLite indices on the underlying
@@ -803,7 +769,6 @@ static char *echoSelectList(echo_vtab *pTab, sqlite3_index_info *pIdxInfo){
 static int echoBestIndex(sqlite3_vtab *tab, sqlite3_index_info *pIdxInfo){
   int ii;
   char *zQuery = 0;
-  char *zCol = 0;
   char *zNew;
   int nArg = 0;
   const char *zSep = "WHERE";
@@ -811,11 +776,11 @@ static int echoBestIndex(sqlite3_vtab *tab, sqlite3_index_info *pIdxInfo){
   sqlite3_stmt *pStmt = 0;
   Tcl_Interp *interp = pVtab->interp;
 
-  int nRow = 0;
+  int nRow;
   int useIdx = 0;
   int rc = SQLITE_OK;
   int useCost = 0;
-  double cost = 0;
+  double cost;
   int isIgnoreUsable = 0;
   if( Tcl_GetVar(interp, "echo_module_ignore_usable", TCL_GLOBAL_ONLY) ){
     isIgnoreUsable = 1;
@@ -851,11 +816,10 @@ static int echoBestIndex(sqlite3_vtab *tab, sqlite3_index_info *pIdxInfo){
     }
   }
 
-  zCol = echoSelectList(pVtab, pIdxInfo);
-  if( !zCol ) return SQLITE_NOMEM;
-  zQuery = sqlite3_mprintf("SELECT rowid%z FROM %Q", zCol, pVtab->zTableName);
-  if( !zQuery ) return SQLITE_NOMEM;
-
+  zQuery = sqlite3_mprintf("SELECT rowid, * FROM %Q", pVtab->zTableName);
+  if( !zQuery ){
+    return SQLITE_NOMEM;
+  }
   for(ii=0; ii<pIdxInfo->nConstraint; ii++){
     const struct sqlite3_index_constraint *pConstraint;
     struct sqlite3_index_constraint_usage *pUsage;
@@ -867,10 +831,13 @@ static int echoBestIndex(sqlite3_vtab *tab, sqlite3_index_info *pIdxInfo){
     if( !isIgnoreUsable && !pConstraint->usable ) continue;
 
     iCol = pConstraint->iColumn;
-    if( iCol<0 || pVtab->aIndex[iCol] ){
-      char *zNewCol = iCol>=0 ? pVtab->aCol[iCol] : "rowid";
+    if( pVtab->aIndex[iCol] || iCol<0 ){
+      char *zCol = pVtab->aCol[iCol];
       char *zOp = 0;
       useIdx = 1;
+      if( iCol<0 ){
+        zCol = "rowid";
+      }
       switch( pConstraint->op ){
         case SQLITE_INDEX_CONSTRAINT_EQ:
           zOp = "="; break;
@@ -883,33 +850,19 @@ static int echoBestIndex(sqlite3_vtab *tab, sqlite3_index_info *pIdxInfo){
         case SQLITE_INDEX_CONSTRAINT_GE:
           zOp = ">="; break;
         case SQLITE_INDEX_CONSTRAINT_MATCH:
-          /* Purposely translate the MATCH operator into a LIKE, which
-          ** will be used by the next block of code to construct a new
-          ** query.  It should also be noted here that the next block
-          ** of code requires the first letter of this operator to be
-          ** in upper-case to trigger the special MATCH handling (i.e.
-          ** wrapping the bound parameter with literal '%'s).
-          */
           zOp = "LIKE"; break;
-        case SQLITE_INDEX_CONSTRAINT_LIKE:
-          zOp = "like"; break;
-        case SQLITE_INDEX_CONSTRAINT_GLOB:
-          zOp = "glob"; break;
-        case SQLITE_INDEX_CONSTRAINT_REGEXP:
-          zOp = "regexp"; break;
       }
-      if( zOp ){
-        if( zOp[0]=='L' ){
-          zNew = sqlite3_mprintf(" %s %s LIKE (SELECT '%%'||?||'%%')", 
-              zSep, zNewCol);
-        } else {
-          zNew = sqlite3_mprintf(" %s %s %s ?", zSep, zNewCol, zOp);
-        }
-        string_concat(&zQuery, zNew, 1, &rc);
-        zSep = "AND";
-        pUsage->argvIndex = ++nArg;
-        pUsage->omit = 1;
+      if( zOp[0]=='L' ){
+        zNew = sqlite3_mprintf(" %s %s LIKE (SELECT '%%'||?||'%%')", 
+                               zSep, zCol);
+      } else {
+        zNew = sqlite3_mprintf(" %s %s %s ?", zSep, zCol, zOp);
       }
+      string_concat(&zQuery, zNew, 1, &rc);
+
+      zSep = "AND";
+      pUsage->argvIndex = ++nArg;
+      pUsage->omit = 1;
     }
   }
 
@@ -917,13 +870,14 @@ static int echoBestIndex(sqlite3_vtab *tab, sqlite3_index_info *pIdxInfo){
   ** on a column that this virtual table has an index for, then consume 
   ** the ORDER BY clause.
   */
-  if( pIdxInfo->nOrderBy==1 && (
-        pIdxInfo->aOrderBy->iColumn<0 ||
-        pVtab->aIndex[pIdxInfo->aOrderBy->iColumn]) ){
+  if( pIdxInfo->nOrderBy==1 && pVtab->aIndex[pIdxInfo->aOrderBy->iColumn] ){
     int iCol = pIdxInfo->aOrderBy->iColumn;
-    char *zNewCol = iCol>=0 ? pVtab->aCol[iCol] : "rowid";
+    char *zCol = pVtab->aCol[iCol];
     char *zDir = pIdxInfo->aOrderBy->desc?"DESC":"ASC";
-    zNew = sqlite3_mprintf(" ORDER BY %s %s", zNewCol, zDir);
+    if( iCol<0 ){
+      zCol = "rowid";
+    }
+    zNew = sqlite3_mprintf(" ORDER BY %s %s", zCol, zDir);
     string_concat(&zQuery, zNew, 1, &rc);
     pIdxInfo->orderByConsumed = 1;
   }
@@ -941,7 +895,7 @@ static int echoBestIndex(sqlite3_vtab *tab, sqlite3_index_info *pIdxInfo){
     pIdxInfo->estimatedCost = cost;
   }else if( useIdx ){
     /* Approximation of log2(nRow). */
-    for( ii=0; ii<(sizeof(int)*8)-1; ii++ ){
+    for( ii=0; ii<(sizeof(int)*8); ii++ ){
       if( nRow & (1<<ii) ){
         pIdxInfo->estimatedCost = (double)ii;
       }
@@ -976,7 +930,7 @@ int echoUpdate(
   sqlite3 *db = pVtab->db;
   int rc = SQLITE_OK;
 
-  sqlite3_stmt *pStmt = 0;
+  sqlite3_stmt *pStmt;
   char *z = 0;               /* SQL statement to execute */
   int bindArgZero = 0;       /* True to bind apData[0] to sql var no. nData */
   int bindArgOne = 0;        /* True to bind apData[1] to sql var no. 1 */
@@ -1267,7 +1221,7 @@ static int echoRename(sqlite3_vtab *vtab, const char *zNewName){
   }
 
   if( p->isPattern ){
-    int nThis = (int)strlen(p->zThis);
+    int nThis = strlen(p->zThis);
     char *zSql = sqlite3_mprintf("ALTER TABLE %s RENAME TO %s%s", 
         p->zTableName, zNewName, &p->zTableName[nThis]
     );
@@ -1278,27 +1232,12 @@ static int echoRename(sqlite3_vtab *vtab, const char *zNewName){
   return rc;
 }
 
-static int echoSavepoint(sqlite3_vtab *pVTab, int iSavepoint){
-  assert( pVTab );
-  return SQLITE_OK;
-}
-
-static int echoRelease(sqlite3_vtab *pVTab, int iSavepoint){
-  assert( pVTab );
-  return SQLITE_OK;
-}
-
-static int echoRollbackTo(sqlite3_vtab *pVTab, int iSavepoint){
-  assert( pVTab );
-  return SQLITE_OK;
-}
-
 /*
 ** A virtual table module that merely "echos" the contents of another
 ** table (like an SQL VIEW).
 */
 static sqlite3_module echoModule = {
-  1,                         /* iVersion */
+  0,                         /* iVersion */
   echoCreate,
   echoConnect,
   echoBestIndex,
@@ -1318,64 +1257,26 @@ static sqlite3_module echoModule = {
   echoRollback,              /* xRollback - rollback transaction */
   echoFindFunction,          /* xFindFunction - function overloading */
   echoRename,                /* xRename - rename the table */
-  0,                         /* xSavepoint */
-  0,                         /* xRelease */
-  0,                         /* xRollbackTo */
-  0,                         /* xShadowName */
-  0                          /* xIntegrity */
-};
-
-static sqlite3_module echoModuleV2 = {
-  2,                         /* iVersion */
-  echoCreate,
-  echoConnect,
-  echoBestIndex,
-  echoDisconnect, 
-  echoDestroy,
-  echoOpen,                  /* xOpen - open a cursor */
-  echoClose,                 /* xClose - close a cursor */
-  echoFilter,                /* xFilter - configure scan constraints */
-  echoNext,                  /* xNext - advance a cursor */
-  echoEof,                   /* xEof */
-  echoColumn,                /* xColumn - read data */
-  echoRowid,                 /* xRowid - read data */
-  echoUpdate,                /* xUpdate - write data */
-  echoBegin,                 /* xBegin - begin transaction */
-  echoSync,                  /* xSync - sync transaction */
-  echoCommit,                /* xCommit - commit transaction */
-  echoRollback,              /* xRollback - rollback transaction */
-  echoFindFunction,          /* xFindFunction - function overloading */
-  echoRename,                /* xRename - rename the table */
-  echoSavepoint,
-  echoRelease,
-  echoRollbackTo,
-  0,                         /* xShadowName */
-  0                          /* xIntegrity  */
 };
 
 /*
 ** Decode a pointer to an sqlite3 object.
 */
 extern int getDbPointer(Tcl_Interp *interp, const char *zA, sqlite3 **ppDb);
-extern const char *sqlite3ErrName(int);
 
 static void moduleDestroy(void *p){
-  EchoModule *pMod = (EchoModule*)p;
-  sqlite3_create_function(pMod->db, "function_that_does_not_exist_0982ma98",
-                          SQLITE_ANY, 1, 0, 0, 0, 0);
   sqlite3_free(p);
 }
 
 /*
 ** Register the echo virtual table module.
 */
-static int SQLITE_TCLAPI register_echo_module(
+static int register_echo_module(
   ClientData clientData, /* Pointer to sqlite3_enable_XXX function */
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int objc,              /* Number of arguments */
   Tcl_Obj *CONST objv[]  /* Command arguments */
 ){
-  int rc;
   sqlite3 *db;
   EchoModule *pMod;
   if( objc!=2 ){
@@ -1383,26 +1284,9 @@ static int SQLITE_TCLAPI register_echo_module(
     return TCL_ERROR;
   }
   if( getDbPointer(interp, Tcl_GetString(objv[1]), &db) ) return TCL_ERROR;
-
-  /* Virtual table module "echo" */
   pMod = sqlite3_malloc(sizeof(EchoModule));
   pMod->interp = interp;
-  pMod->db = db;
-  rc = sqlite3_create_module_v2(
-      db, "echo", &echoModule, (void*)pMod, moduleDestroy
-  );
-
-  /* Virtual table module "echo_v2" */
-  if( rc==SQLITE_OK ){
-    pMod = sqlite3_malloc(sizeof(EchoModule));
-    pMod->interp = interp;
-    pMod->db = db;
-    rc = sqlite3_create_module_v2(db, "echo_v2", 
-        &echoModuleV2, (void*)pMod, moduleDestroy
-    );
-  }
-
-  Tcl_SetResult(interp, (char *)sqlite3ErrName(rc), TCL_STATIC);
+  sqlite3_create_module_v2(db, "echo", &echoModule, (void*)pMod, moduleDestroy);
   return TCL_OK;
 }
 
@@ -1411,7 +1295,7 @@ static int SQLITE_TCLAPI register_echo_module(
 **
 ** sqlite3_declare_vtab DB SQL
 */
-static int SQLITE_TCLAPI declare_vtab(
+static int declare_vtab(
   ClientData clientData, /* Pointer to sqlite3_enable_XXX function */
   Tcl_Interp *interp,    /* The TCL interpreter that invoked this command */
   int objc,              /* Number of arguments */
@@ -1444,8 +1328,8 @@ int Sqlitetest8_Init(Tcl_Interp *interp){
      Tcl_ObjCmdProc *xProc;
      void *clientData;
   } aObjCmd[] = {
-     { "register_echo_module",       register_echo_module, 0 },
-     { "sqlite3_declare_vtab",       declare_vtab, 0 },
+     { "register_echo_module",   register_echo_module, 0 },
+     { "sqlite3_declare_vtab",   declare_vtab, 0 },
   };
   int i;
   for(i=0; i<sizeof(aObjCmd)/sizeof(aObjCmd[0]); i++){
