@@ -198,8 +198,7 @@ static fs_vfs_t fs_vfs = {
     fsDlClose,                                  /* xDlClose */
     fsRandomness,                               /* xRandomness */
     fsSleep,                                    /* xSleep */
-    fsCurrentTime,                              /* xCurrentTime */
-    0                                           /* xCurrentTimeInt64 */
+    fsCurrentTime                               /* xCurrentTime */
   }, 
   0,                                            /* pFileList */
   0                                             /* pParent */
@@ -218,11 +217,7 @@ static sqlite3_io_methods fs_io_methods = {
   fsCheckReservedLock,          /* xCheckReservedLock */
   fsFileControl,                /* xFileControl */
   fsSectorSize,                 /* xSectorSize */
-  fsDeviceCharacteristics,      /* xDeviceCharacteristics */
-  0,                            /* xShmMap */
-  0,                            /* xShmLock */
-  0,                            /* xShmBarrier */
-  0                             /* xShmUnmap */
+  fsDeviceCharacteristics       /* xDeviceCharacteristics */
 };
 
 
@@ -239,11 +234,7 @@ static sqlite3_io_methods tmp_io_methods = {
   tmpCheckReservedLock,         /* xCheckReservedLock */
   tmpFileControl,               /* xFileControl */
   tmpSectorSize,                /* xSectorSize */
-  tmpDeviceCharacteristics,     /* xDeviceCharacteristics */
-  0,                            /* xShmMap */
-  0,                            /* xShmLock */
-  0,                            /* xShmBarrier */
-  0                             /* xShmUnmap */
+  tmpDeviceCharacteristics      /* xDeviceCharacteristics */
 };
 
 /* Useful macros used in several places */
@@ -288,7 +279,7 @@ static int tmpWrite(
 ){
   tmp_file *pTmp = (tmp_file *)pFile;
   if( (iAmt+iOfst)>pTmp->nAlloc ){
-    int nNew = (int)(2*(iAmt+iOfst+pTmp->nAlloc));
+    int nNew = 2*(iAmt+iOfst+pTmp->nAlloc);
     char *zNew = sqlite3_realloc(pTmp->zAlloc, nNew);
     if( !zNew ){
       return SQLITE_NOMEM;
@@ -297,7 +288,7 @@ static int tmpWrite(
     pTmp->nAlloc = nNew;
   }
   memcpy(&pTmp->zAlloc[iOfst], zBuf, iAmt);
-  pTmp->nSize = (int)MAX(pTmp->nSize, iOfst+iAmt);
+  pTmp->nSize = MAX(pTmp->nSize, iOfst+iAmt);
   return SQLITE_OK;
 }
 
@@ -306,7 +297,7 @@ static int tmpWrite(
 */
 static int tmpTruncate(sqlite3_file *pFile, sqlite_int64 size){
   tmp_file *pTmp = (tmp_file *)pFile;
-  pTmp->nSize = (int)MIN(pTmp->nSize, size);
+  pTmp->nSize = MIN(pTmp->nSize, size);
   return SQLITE_OK;
 }
 
@@ -418,7 +409,7 @@ static int fsRead(
     /* Journal file. */
     int iRem = iAmt;
     int iBuf = 0;
-    int ii = (int)iOfst;
+    int ii = iOfst;
     while( iRem>0 && rc==SQLITE_OK ){
       int iRealOff = pReal->nBlob - BLOCKSIZE*((ii/BLOCKSIZE)+1) + ii%BLOCKSIZE;
       int iRealAmt = MIN(iRem, BLOCKSIZE - (iRealOff%BLOCKSIZE));
@@ -453,14 +444,14 @@ static int fsWrite(
     }else{
       rc = pF->pMethods->xWrite(pF, zBuf, iAmt, iOfst+BLOCKSIZE);
       if( rc==SQLITE_OK ){
-        pReal->nDatabase = (int)MAX(pReal->nDatabase, iAmt+iOfst);
+        pReal->nDatabase = MAX(pReal->nDatabase, iAmt+iOfst);
       }
     }
   }else{
     /* Journal file. */
     int iRem = iAmt;
     int iBuf = 0;
-    int ii = (int)iOfst;
+    int ii = iOfst;
     while( iRem>0 && rc==SQLITE_OK ){
       int iRealOff = pReal->nBlob - BLOCKSIZE*((ii/BLOCKSIZE)+1) + ii%BLOCKSIZE;
       int iRealAmt = MIN(iRem, BLOCKSIZE - (iRealOff%BLOCKSIZE));
@@ -475,7 +466,7 @@ static int fsWrite(
       }
     }
     if( rc==SQLITE_OK ){
-      pReal->nJournal = (int)MAX(pReal->nJournal, iAmt+iOfst);
+      pReal->nJournal = MAX(pReal->nJournal, iAmt+iOfst);
     }
   }
 
@@ -489,9 +480,9 @@ static int fsTruncate(sqlite3_file *pFile, sqlite_int64 size){
   fs_file *p = (fs_file *)pFile;
   fs_real_file *pReal = p->pReal;
   if( p->eType==DATABASE_FILE ){
-    pReal->nDatabase = (int)MIN(pReal->nDatabase, size);
+    pReal->nDatabase = MIN(pReal->nDatabase, size);
   }else{
-    pReal->nJournal = (int)MIN(pReal->nJournal, size);
+    pReal->nJournal = MIN(pReal->nJournal, size);
   }
   return SQLITE_OK;
 }
@@ -595,9 +586,9 @@ static int fsOpen(
   int rc = SQLITE_OK;
 
   if( 0==(flags&(SQLITE_OPEN_MAIN_DB|SQLITE_OPEN_MAIN_JOURNAL)) ){
-    tmp_file *p2 = (tmp_file *)pFile;
-    memset(p2, 0, sizeof(*p2));
-    p2->base.pMethods = &tmp_io_methods;
+    tmp_file *p = (tmp_file *)pFile;
+    memset(p, 0, sizeof(*p));
+    p->base.pMethods = &tmp_io_methods;
     return SQLITE_OK;
   }
 
@@ -606,7 +597,7 @@ static int fsOpen(
   p->eType = eType;
 
   assert(strlen("-journal")==8);
-  nName = (int)strlen(zName)-((eType==JOURNAL_FILE)?8:0);
+  nName = strlen(zName)-((eType==JOURNAL_FILE)?8:0);
   pReal=pFsVfs->pFileList; 
   for(; pReal && strncmp(pReal->zName, zName, nName); pReal=pReal->pNext);
 
@@ -641,7 +632,7 @@ static int fsOpen(
       pReal->nBlob = BLOBSIZE;
     }else{
       unsigned char zS[4];
-      pReal->nBlob = (int)size;
+      pReal->nBlob = size;
       rc = pRealFile->pMethods->xRead(pRealFile, zS, 4, 0);
       pReal->nDatabase = (zS[0]<<24)+(zS[1]<<16)+(zS[2]<<8)+zS[3];
       if( rc==SQLITE_OK ){
@@ -687,7 +678,7 @@ static int fsDelete(sqlite3_vfs *pVfs, const char *zPath, int dirSync){
   fs_vfs_t *pFsVfs = (fs_vfs_t *)pVfs;
   fs_real_file *pReal;
   sqlite3_file *pF;
-  int nName = (int)strlen(zPath) - 8;
+  int nName = strlen(zPath) - 8;
 
   assert(strlen("-journal")==8);
   assert(strcmp("-journal", &zPath[nName])==0);
@@ -717,7 +708,7 @@ static int fsAccess(
   fs_vfs_t *pFsVfs = (fs_vfs_t *)pVfs;
   fs_real_file *pReal;
   int isJournal = 0;
-  int nName = (int)strlen(zPath);
+  int nName = strlen(zPath);
 
   if( flags!=SQLITE_ACCESS_EXISTS ){
     sqlite3_vfs *pParent = ((fs_vfs_t *)pVfs)->pParent;
